# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class TableDataPreview(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'table_schema': 'list[ColumnInfo]',
        'data_array': 'list[list[object]]',
        'row_count': 'int',
        'truncated': 'bool'
    }

    attribute_map = {
        'table_schema': 'table_schema',
        'data_array': 'data_array',
        'row_count': 'row_count',
        'truncated': 'truncated'
    }

    def __init__(self, table_schema=None, data_array=None, row_count=None, truncated=False, local_vars_configuration=None):  # noqa: E501
        """TableDataPreview - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._table_schema = None
        self._data_array = None
        self._row_count = None
        self._truncated = None
        self.discriminator = None

        self.table_schema = table_schema
        self.data_array = data_array
        self.row_count = row_count
        if truncated is not None:
            self.truncated = truncated

    @property
    def table_schema(self):
        """Gets the table_schema of this TableDataPreview.  # noqa: E501

        Schema definition with column information  # noqa: E501

        :return: The table_schema of this TableDataPreview.  # noqa: E501
        :rtype: list[ColumnInfo]
        """
        return self._table_schema

    @table_schema.setter
    def table_schema(self, table_schema):
        """Sets the table_schema of this TableDataPreview.

        Schema definition with column information  # noqa: E501

        :param table_schema: The table_schema of this TableDataPreview.  # noqa: E501
        :type: list[ColumnInfo]
        """
        if self.local_vars_configuration.client_side_validation and table_schema is None:  # noqa: E501
            raise ValueError("Invalid value for `table_schema`, must not be `None`")  # noqa: E501

        self._table_schema = table_schema

    @property
    def data_array(self):
        """Gets the data_array of this TableDataPreview.  # noqa: E501

        Preview data as array of rows, where each row is an array of values  # noqa: E501

        :return: The data_array of this TableDataPreview.  # noqa: E501
        :rtype: list[list[object]]
        """
        return self._data_array

    @data_array.setter
    def data_array(self, data_array):
        """Sets the data_array of this TableDataPreview.

        Preview data as array of rows, where each row is an array of values  # noqa: E501

        :param data_array: The data_array of this TableDataPreview.  # noqa: E501
        :type: list[list[object]]
        """
        if self.local_vars_configuration.client_side_validation and data_array is None:  # noqa: E501
            raise ValueError("Invalid value for `data_array`, must not be `None`")  # noqa: E501

        self._data_array = data_array

    @property
    def row_count(self):
        """Gets the row_count of this TableDataPreview.  # noqa: E501

        Number of rows returned in this preview  # noqa: E501

        :return: The row_count of this TableDataPreview.  # noqa: E501
        :rtype: int
        """
        return self._row_count

    @row_count.setter
    def row_count(self, row_count):
        """Sets the row_count of this TableDataPreview.

        Number of rows returned in this preview  # noqa: E501

        :param row_count: The row_count of this TableDataPreview.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and row_count is None:  # noqa: E501
            raise ValueError("Invalid value for `row_count`, must not be `None`")  # noqa: E501

        self._row_count = row_count

    @property
    def truncated(self):
        """Gets the truncated of this TableDataPreview.  # noqa: E501

        True if more rows exist in the table than are returned in this preview  # noqa: E501

        :return: The truncated of this TableDataPreview.  # noqa: E501
        :rtype: bool
        """
        return self._truncated

    @truncated.setter
    def truncated(self, truncated):
        """Sets the truncated of this TableDataPreview.

        True if more rows exist in the table than are returned in this preview  # noqa: E501

        :param truncated: The truncated of this TableDataPreview.  # noqa: E501
        :type: bool
        """

        self._truncated = truncated

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TableDataPreview):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, TableDataPreview):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class CreditRecordCommitV2(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'credit_name': 'str',
        'contract_name': 'str',
        'effective_date_start': 'date',
        'effective_date_end': 'date',
        'total_balance_usd': 'float',
        'last_month_usage_usd': 'float',
        'amount_consumed_usd': 'float',
        'total_granted_usd': 'float',
        'ledger': 'list[CommitLedgerRecordV2]',
        'grants': 'list[CreditGrantRecordV2]',
        'credit_type': 'CreditType'
    }

    attribute_map = {
        'credit_name': 'credit_name',
        'contract_name': 'contract_name',
        'effective_date_start': 'effective_date_start',
        'effective_date_end': 'effective_date_end',
        'total_balance_usd': 'total_balance_usd',
        'last_month_usage_usd': 'last_month_usage_usd',
        'amount_consumed_usd': 'amount_consumed_usd',
        'total_granted_usd': 'total_granted_usd',
        'ledger': 'ledger',
        'grants': 'grants',
        'credit_type': 'credit_type'
    }

    def __init__(self, credit_name=None, contract_name=None, effective_date_start=None, effective_date_end=None, total_balance_usd=None, last_month_usage_usd=None, amount_consumed_usd=None, total_granted_usd=None, ledger=None, grants=None, credit_type=None, local_vars_configuration=None):  # noqa: E501
        """CreditRecordCommitV2 - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._credit_name = None
        self._contract_name = None
        self._effective_date_start = None
        self._effective_date_end = None
        self._total_balance_usd = None
        self._last_month_usage_usd = None
        self._amount_consumed_usd = None
        self._total_granted_usd = None
        self._ledger = None
        self._grants = None
        self._credit_type = None
        self.discriminator = None

        self.credit_name = credit_name
        self.contract_name = contract_name
        self.effective_date_start = effective_date_start
        self.effective_date_end = effective_date_end
        self.total_balance_usd = total_balance_usd
        self.last_month_usage_usd = last_month_usage_usd
        self.amount_consumed_usd = amount_consumed_usd
        self.total_granted_usd = total_granted_usd
        self.ledger = ledger
        self.grants = grants
        if credit_type is not None:
            self.credit_type = credit_type

    @property
    def credit_name(self):
        """Gets the credit_name of this CreditRecordCommitV2.  # noqa: E501

        Name of the credit.  # noqa: E501

        :return: The credit_name of this CreditRecordCommitV2.  # noqa: E501
        :rtype: str
        """
        return self._credit_name

    @credit_name.setter
    def credit_name(self, credit_name):
        """Sets the credit_name of this CreditRecordCommitV2.

        Name of the credit.  # noqa: E501

        :param credit_name: The credit_name of this CreditRecordCommitV2.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and credit_name is None:  # noqa: E501
            raise ValueError("Invalid value for `credit_name`, must not be `None`")  # noqa: E501

        self._credit_name = credit_name

    @property
    def contract_name(self):
        """Gets the contract_name of this CreditRecordCommitV2.  # noqa: E501

        Name of the contract the credit is attached to.  # noqa: E501

        :return: The contract_name of this CreditRecordCommitV2.  # noqa: E501
        :rtype: str
        """
        return self._contract_name

    @contract_name.setter
    def contract_name(self, contract_name):
        """Sets the contract_name of this CreditRecordCommitV2.

        Name of the contract the credit is attached to.  # noqa: E501

        :param contract_name: The contract_name of this CreditRecordCommitV2.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and contract_name is None:  # noqa: E501
            raise ValueError("Invalid value for `contract_name`, must not be `None`")  # noqa: E501

        self._contract_name = contract_name

    @property
    def effective_date_start(self):
        """Gets the effective_date_start of this CreditRecordCommitV2.  # noqa: E501

        UTC date the credit can first be used (inclusive).  # noqa: E501

        :return: The effective_date_start of this CreditRecordCommitV2.  # noqa: E501
        :rtype: date
        """
        return self._effective_date_start

    @effective_date_start.setter
    def effective_date_start(self, effective_date_start):
        """Sets the effective_date_start of this CreditRecordCommitV2.

        UTC date the credit can first be used (inclusive).  # noqa: E501

        :param effective_date_start: The effective_date_start of this CreditRecordCommitV2.  # noqa: E501
        :type: date
        """
        if self.local_vars_configuration.client_side_validation and effective_date_start is None:  # noqa: E501
            raise ValueError("Invalid value for `effective_date_start`, must not be `None`")  # noqa: E501

        self._effective_date_start = effective_date_start

    @property
    def effective_date_end(self):
        """Gets the effective_date_end of this CreditRecordCommitV2.  # noqa: E501

        UTC date the credit can last be used (inclusive).  # noqa: E501

        :return: The effective_date_end of this CreditRecordCommitV2.  # noqa: E501
        :rtype: date
        """
        return self._effective_date_end

    @effective_date_end.setter
    def effective_date_end(self, effective_date_end):
        """Sets the effective_date_end of this CreditRecordCommitV2.

        UTC date the credit can last be used (inclusive).  # noqa: E501

        :param effective_date_end: The effective_date_end of this CreditRecordCommitV2.  # noqa: E501
        :type: date
        """
        if self.local_vars_configuration.client_side_validation and effective_date_end is None:  # noqa: E501
            raise ValueError("Invalid value for `effective_date_end`, must not be `None`")  # noqa: E501

        self._effective_date_end = effective_date_end

    @property
    def total_balance_usd(self):
        """Gets the total_balance_usd of this CreditRecordCommitV2.  # noqa: E501

        Total balance of the credit in USD.  # noqa: E501

        :return: The total_balance_usd of this CreditRecordCommitV2.  # noqa: E501
        :rtype: float
        """
        return self._total_balance_usd

    @total_balance_usd.setter
    def total_balance_usd(self, total_balance_usd):
        """Sets the total_balance_usd of this CreditRecordCommitV2.

        Total balance of the credit in USD.  # noqa: E501

        :param total_balance_usd: The total_balance_usd of this CreditRecordCommitV2.  # noqa: E501
        :type: float
        """
        if self.local_vars_configuration.client_side_validation and total_balance_usd is None:  # noqa: E501
            raise ValueError("Invalid value for `total_balance_usd`, must not be `None`")  # noqa: E501

        self._total_balance_usd = total_balance_usd

    @property
    def last_month_usage_usd(self):
        """Gets the last_month_usage_usd of this CreditRecordCommitV2.  # noqa: E501

        Amount of USD consumed from the credit in the last month.  # noqa: E501

        :return: The last_month_usage_usd of this CreditRecordCommitV2.  # noqa: E501
        :rtype: float
        """
        return self._last_month_usage_usd

    @last_month_usage_usd.setter
    def last_month_usage_usd(self, last_month_usage_usd):
        """Sets the last_month_usage_usd of this CreditRecordCommitV2.

        Amount of USD consumed from the credit in the last month.  # noqa: E501

        :param last_month_usage_usd: The last_month_usage_usd of this CreditRecordCommitV2.  # noqa: E501
        :type: float
        """
        if self.local_vars_configuration.client_side_validation and last_month_usage_usd is None:  # noqa: E501
            raise ValueError("Invalid value for `last_month_usage_usd`, must not be `None`")  # noqa: E501

        self._last_month_usage_usd = last_month_usage_usd

    @property
    def amount_consumed_usd(self):
        """Gets the amount_consumed_usd of this CreditRecordCommitV2.  # noqa: E501

        Amount of USD consumed from the credit.  # noqa: E501

        :return: The amount_consumed_usd of this CreditRecordCommitV2.  # noqa: E501
        :rtype: float
        """
        return self._amount_consumed_usd

    @amount_consumed_usd.setter
    def amount_consumed_usd(self, amount_consumed_usd):
        """Sets the amount_consumed_usd of this CreditRecordCommitV2.

        Amount of USD consumed from the credit.  # noqa: E501

        :param amount_consumed_usd: The amount_consumed_usd of this CreditRecordCommitV2.  # noqa: E501
        :type: float
        """
        if self.local_vars_configuration.client_side_validation and amount_consumed_usd is None:  # noqa: E501
            raise ValueError("Invalid value for `amount_consumed_usd`, must not be `None`")  # noqa: E501

        self._amount_consumed_usd = amount_consumed_usd

    @property
    def total_granted_usd(self):
        """Gets the total_granted_usd of this CreditRecordCommitV2.  # noqa: E501

        Total amount of USD granted to the credit.  # noqa: E501

        :return: The total_granted_usd of this CreditRecordCommitV2.  # noqa: E501
        :rtype: float
        """
        return self._total_granted_usd

    @total_granted_usd.setter
    def total_granted_usd(self, total_granted_usd):
        """Sets the total_granted_usd of this CreditRecordCommitV2.

        Total amount of USD granted to the credit.  # noqa: E501

        :param total_granted_usd: The total_granted_usd of this CreditRecordCommitV2.  # noqa: E501
        :type: float
        """
        if self.local_vars_configuration.client_side_validation and total_granted_usd is None:  # noqa: E501
            raise ValueError("Invalid value for `total_granted_usd`, must not be `None`")  # noqa: E501

        self._total_granted_usd = total_granted_usd

    @property
    def ledger(self):
        """Gets the ledger of this CreditRecordCommitV2.  # noqa: E501

        List of ledger records for this credit.  # noqa: E501

        :return: The ledger of this CreditRecordCommitV2.  # noqa: E501
        :rtype: list[CommitLedgerRecordV2]
        """
        return self._ledger

    @ledger.setter
    def ledger(self, ledger):
        """Sets the ledger of this CreditRecordCommitV2.

        List of ledger records for this credit.  # noqa: E501

        :param ledger: The ledger of this CreditRecordCommitV2.  # noqa: E501
        :type: list[CommitLedgerRecordV2]
        """
        if self.local_vars_configuration.client_side_validation and ledger is None:  # noqa: E501
            raise ValueError("Invalid value for `ledger`, must not be `None`")  # noqa: E501

        self._ledger = ledger

    @property
    def grants(self):
        """Gets the grants of this CreditRecordCommitV2.  # noqa: E501

        List of grant records for this credit.  # noqa: E501

        :return: The grants of this CreditRecordCommitV2.  # noqa: E501
        :rtype: list[CreditGrantRecordV2]
        """
        return self._grants

    @grants.setter
    def grants(self, grants):
        """Sets the grants of this CreditRecordCommitV2.

        List of grant records for this credit.  # noqa: E501

        :param grants: The grants of this CreditRecordCommitV2.  # noqa: E501
        :type: list[CreditGrantRecordV2]
        """
        if self.local_vars_configuration.client_side_validation and grants is None:  # noqa: E501
            raise ValueError("Invalid value for `grants`, must not be `None`")  # noqa: E501

        self._grants = grants

    @property
    def credit_type(self):
        """Gets the credit_type of this CreditRecordCommitV2.  # noqa: E501


        :return: The credit_type of this CreditRecordCommitV2.  # noqa: E501
        :rtype: CreditType
        """
        return self._credit_type

    @credit_type.setter
    def credit_type(self, credit_type):
        """Sets the credit_type of this CreditRecordCommitV2.


        :param credit_type: The credit_type of this CreditRecordCommitV2.  # noqa: E501
        :type: CreditType
        """

        self._credit_type = credit_type

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CreditRecordCommitV2):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, CreditRecordCommitV2):
            return True

        return self.to_dict() != other.to_dict()



from dataclasses import dataclass, field
from datetime import datetime
from typing import (
    ClassVar,
    Dict,
    get_type_hints,
    Literal,
    Optional,
    TYPE_CHECKING,
    Union,
)

from anyscale._private.models import ModelBase, ModelEnum


class SessionState(ModelEnum):
    """State of a cluster/session."""

    Stopped = "Stopped"
    Terminated = "Terminated"
    StartingUp = "StartingUp"
    StartupErrored = "StartupErrored"
    Running = "Running"
    Updating = "Updating"
    UpdatingErrored = "UpdatingErrored"
    Stopping = "Stopping"
    Terminating = "Terminating"
    AwaitingStartup = "AwaitingStartup"
    AwaitingFileMounts = "AwaitingFileMounts"
    TerminatingErrored = "TerminatingErrored"
    StoppingErrored = "StoppingErrored"

    __docstrings__: ClassVar[Dict[str, str]] = {
        "Stopped": "The cluster is stopped.",
        "Terminated": "The cluster is terminated.",
        "StartingUp": "The cluster is starting up.",
        "StartupErrored": "The cluster encountered an error during startup.",
        "Running": "The cluster is running.",
        "Updating": "The cluster is being updated.",
        "UpdatingErrored": "The cluster encountered an error during update.",
        "Stopping": "The cluster is stopping.",
        "Terminating": "The cluster is terminating.",
        "AwaitingStartup": "The cluster is awaiting startup.",
        "AwaitingFileMounts": "The cluster is awaiting file mounts.",
        "TerminatingErrored": "The cluster encountered an error during termination.",
        "StoppingErrored": "The cluster encountered an error while stopping.",
    }


class JobQueueState(ModelEnum):
    """Current state of a job queue."""

    ACTIVE = "ACTIVE"
    SEALED = "SEALED"
    # Add other potential states if necessary based on API reality
    UNKNOWN = "UNKNOWN"

    __docstrings__: ClassVar[Dict[str, str]] = {
        ACTIVE: "The job queue is active and accepting jobs.",
        SEALED: "The job queue is sealed and not accepting new jobs. It may still be processing existing jobs.",
        UNKNOWN: "The state of the job queue is unknown or could not be determined.",
    }


class SortOrder(ModelEnum):
    """Sort order for queries."""

    ASC = "ASC"
    DESC = "DESC"

    __docstrings__: ClassVar[Dict[str, str]] = {
        ASC: "Ascending order.",
        DESC: "Descending order.",
    }


class JobQueueSortField(ModelEnum):
    """Fields available for sorting job queues."""

    ID = "ID"
    NAME = "NAME"
    CREATED_AT = "CREATED_AT"
    CREATOR_ID = "CREATOR_ID"
    CREATOR_EMAIL = "CREATOR_EMAIL"
    PROJECT_ID = "PROJECT_ID"
    CLOUD_ID = "CLOUD_ID"
    QUEUE_STATE = "QUEUE_STATE"
    CLUSTER_STATE = "CLUSTER_STATE"

    __docstrings__: ClassVar[Dict[str, str]] = {
        ID: "Sort by Job Queue ID.",
        NAME: "Sort by Job Queue name.",
        CREATED_AT: "Sort by creation timestamp.",
        CREATOR_ID: "Sort by the ID of the creator.",
        CREATOR_EMAIL: "Sort by the email of the creator.",
        PROJECT_ID: "Sort by the Project ID.",
        CLOUD_ID: "Sort by the Cloud ID.",
        QUEUE_STATE: "Sort by the Job Queue's state (ACTIVE, SEALED).",
        CLUSTER_STATE: "Sort by the state of the associated cluster.",
    }


class ExecutionMode(ModelEnum):
    """Execution mode of a job queue."""

    FIFO = "FIFO"
    LIFO = "LIFO"
    PRIORITY = "PRIORITY"
    # Add other execution modes as needed
    UNKNOWN = "UNKNOWN"

    __docstrings__: ClassVar[Dict[str, str]] = {
        FIFO: "FIFO execution mode.",
        LIFO: "LIFO execution mode.",
        PRIORITY: "Priority-based execution mode.",
        UNKNOWN: "Unknown execution mode.",
    }


class ClusterState(ModelEnum):
    """Possible states for a cluster."""

    RUNNING = "RUNNING"
    TERMINATED = "TERMINATED"
    PENDING = "PENDING"
    # Add other states as needed
    UNKNOWN = "UNKNOWN"

    __docstrings__: ClassVar[Dict[str, str]] = {
        RUNNING: "The cluster is running.",
        TERMINATED: "The cluster is terminated.",
        PENDING: "The cluster is pending creation.",
        UNKNOWN: "The state of the cluster is unknown.",
    }


@dataclass(frozen=True)
class JobQueueSortDirective(ModelBase):
    """Directive for sorting job queue results."""

    __doc_py_example__ = """
from anyscale.job_queue.models import JobQueueSortDirective, JobQueueSortField, SortOrder

# Create a sort directive
sort_directive = JobQueueSortDirective(
    sort_field=JobQueueSortField.CREATED_AT,
    sort_order=SortOrder.DESC
)
"""

    sort_field: Union[JobQueueSortField, str] = field(
        metadata={"docstring": "The field to sort by."}
    )
    sort_order: Union[SortOrder, str] = field(
        metadata={"docstring": "The sort order (ASC or DESC)."}
    )

    def _validate_sort_field(
        self, sort_field: Union[JobQueueSortField, str]
    ) -> JobQueueSortField:
        return JobQueueSortField.validate(sort_field)

    def _validate_sort_order(self, sort_order: Union[SortOrder, str]) -> SortOrder:
        return SortOrder.validate(sort_order)


@dataclass(frozen=True)
class JobQueueStatus(ModelBase):
    """Represents the status and details of a Job Queue."""

    # Example shown in docs
    __doc_py_example__ = """\
from anyscale.job_queue.models import JobQueueStatus

status = JobQueueStatus(
    id="jq_123",
    state="ACTIVE",
    name="my-queue",
    max_concurrency=5,
    idle_timeout_s=300,
)
"""

    id: str = field(metadata={"docstring": "Unique ID of the job queue."})
    state: str = field(metadata={"docstring": "Current state of the job queue."})
    name: Optional[str] = field(
        default=None, metadata={"docstring": "Name of the job queue."}
    )
    creator_email: Optional[str] = field(
        default=None,
        metadata={"docstring": "Email of the user who created the job queue."},
    )
    project_id: Optional[str] = field(
        default=None,
        metadata={"docstring": "ID of the project this job queue belongs to."},
    )
    created_at: Optional[datetime] = field(
        default=None,
        metadata={"docstring": "Timestamp when the job queue was created."},
    )
    max_concurrency: Optional[int] = field(
        default=None,
        metadata={"docstring": "Maximum number of jobs allowed to run concurrently."},
    )
    idle_timeout_s: Optional[int] = field(
        default=None,
        metadata={
            "docstring": "Idle timeout in seconds before the queue's cluster may shut down."
        },
    )
    user_provided_id: Optional[str] = field(
        default=None,
        metadata={"docstring": "User provided identifier of the job queue."},
    )
    execution_mode: Optional[str] = field(
        default=None, metadata={"docstring": "The execution mode of the job queue."}
    )
    creator_id: Optional[str] = field(
        default=None,
        metadata={"docstring": "Identifier of user who created the job queue."},
    )
    cloud_id: Optional[str] = field(
        default=None,
        metadata={"docstring": "The cloud ID associated with the job queue."},
    )
    total_jobs: Optional[int] = field(
        default=None, metadata={"docstring": "Total number of jobs in the job queue."},
    )
    successful_jobs: Optional[int] = field(
        default=None,
        metadata={"docstring": "Number of successful jobs in the job queue."},
    )
    failed_jobs: Optional[int] = field(
        default=None, metadata={"docstring": "Number of failed jobs in the job queue."},
    )
    active_jobs: Optional[int] = field(
        default=None, metadata={"docstring": "Number of active jobs in the job queue."},
    )

    def _validate_id(self, id: str) -> str:  # noqa: A002
        if not isinstance(id, str) or not id:
            raise ValueError("'id' must be a non-empty string.")
        return id

    def _validate_name(self, name: Optional[str]) -> Optional[str]:
        if name is not None and not isinstance(name, str):
            raise ValueError("'name' must be a string or None.")
        return name

    def _validate_state(self, state: str) -> str:
        validated = JobQueueState.validate(state)
        return validated.value

    def _validate_creator_email(self, creator_email: Optional[str]) -> Optional[str]:
        if creator_email is not None and not isinstance(creator_email, str):
            raise ValueError("'creator_email' must be a string or None.")
        return creator_email

    def _validate_project_id(self, project_id: Optional[str]) -> Optional[str]:
        if project_id is not None and not isinstance(project_id, str):
            raise ValueError("'project_id' must be a string or None.")
        return project_id

    def _validate_created_at(
        self, created_at: Optional[datetime]
    ) -> Optional[datetime]:
        if created_at is not None and not isinstance(created_at, datetime):
            raise ValueError("'created_at' must be a datetime object or None.")
        return created_at

    def _validate_max_concurrency(
        self, max_concurrency: Optional[int]
    ) -> Optional[int]:
        if max_concurrency is not None:
            if not isinstance(max_concurrency, int):
                raise ValueError("'max_concurrency' must be an integer or None.")
            if max_concurrency < 0:
                raise ValueError("'max_concurrency' cannot be negative.")
        return max_concurrency

    def _validate_idle_timeout_s(self, idle_timeout_s: Optional[int]) -> Optional[int]:
        if idle_timeout_s is not None:
            if not isinstance(idle_timeout_s, int):
                raise ValueError("'idle_timeout_s' must be an integer or None.")
            if idle_timeout_s < 0:
                raise ValueError("'idle_timeout_s' cannot be negative.")
        return idle_timeout_s

    def _validate_user_provided_id(
        self, user_provided_id: Optional[str]
    ) -> Optional[str]:
        if user_provided_id is not None and not isinstance(user_provided_id, str):
            raise ValueError("'user_provided_id' must be a string or None.")
        return user_provided_id

    def _validate_execution_mode(self, execution_mode: Optional[str]) -> Optional[str]:
        if execution_mode is not None:
            validated = ExecutionMode.validate(execution_mode)
            return validated.value
        return None

    def _validate_creator_id(self, creator_id: Optional[str]) -> Optional[str]:
        if creator_id is not None and not isinstance(creator_id, str):
            raise ValueError("'creator_id' must be a string or None.")
        return creator_id

    def _validate_cluster_id(self, cluster_id: Optional[str]) -> Optional[str]:
        if cluster_id is not None and not isinstance(cluster_id, str):
            raise ValueError("'cluster_id' must be a string or None.")
        return cluster_id

    def _validate_current_cluster_state(
        self, current_cluster_state: Optional[Union[ClusterState, str]]
    ) -> Optional[ClusterState]:
        if current_cluster_state is not None:
            return ClusterState.validate(current_cluster_state)
        return None

    def _validate_cloud_id(self, cloud_id: Optional[str]) -> Optional[str]:
        if cloud_id is not None and not isinstance(cloud_id, str):
            raise ValueError("'cloud_id' must be a string or None.")
        return cloud_id

    def _validate_total_jobs(self, total_jobs: Optional[int]) -> Optional[int]:
        if total_jobs is not None:
            if not isinstance(total_jobs, int):
                raise ValueError("'total_jobs' must be an integer or None.")
            if total_jobs < 0:
                raise ValueError("'total_jobs' cannot be negative.")
        return total_jobs

    def _validate_successful_jobs(
        self, successful_jobs: Optional[int]
    ) -> Optional[int]:
        if successful_jobs is not None:
            if not isinstance(successful_jobs, int):
                raise ValueError("'successful_jobs' must be an integer or None.")
            if successful_jobs < 0:
                raise ValueError("'successful_jobs' cannot be negative.")
        return successful_jobs

    def _validate_failed_jobs(self, failed_jobs: Optional[int]) -> Optional[int]:
        if failed_jobs is not None:
            if not isinstance(failed_jobs, int):
                raise ValueError("'failed_jobs' must be an integer or None.")
            if failed_jobs < 0:
                raise ValueError("'failed_jobs' cannot be negative.")
        return failed_jobs

    def _validate_active_jobs(self, active_jobs: Optional[int]) -> Optional[int]:
        if active_jobs is not None:
            if not isinstance(active_jobs, int):
                raise ValueError("'active_jobs' must be an integer or None.")
            if active_jobs < 0:
                raise ValueError("'active_jobs' cannot be negative.")
        return active_jobs


if TYPE_CHECKING:
    JobQueueStatusKeys = Literal[
        "active_jobs",
        "cloud_id",
        "created_at",
        "creator_email",
        "creator_id",
        "execution_mode",
        "failed_jobs",
        "id",
        "idle_timeout_s",
        "max_concurrency",
        "name",
        "project_id",
        "state",
        "successful_jobs",
        "total_jobs",
        "user_provided_id",
    ]
else:
    JobQueueStatusKeys = Literal[tuple(get_type_hints(JobQueueStatus).keys())]



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class ColumnInfo(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'name': 'str',
        'type_text': 'str',
        'type_name': 'str',
        'description': 'str',
        'nullable': 'bool',
        'position': 'int'
    }

    attribute_map = {
        'name': 'name',
        'type_text': 'type_text',
        'type_name': 'type_name',
        'description': 'description',
        'nullable': 'nullable',
        'position': 'position'
    }

    def __init__(self, name=None, type_text=None, type_name=None, description=None, nullable=True, position=0, local_vars_configuration=None):  # noqa: E501
        """ColumnInfo - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._name = None
        self._type_text = None
        self._type_name = None
        self._description = None
        self._nullable = None
        self._position = None
        self.discriminator = None

        self.name = name
        self.type_text = type_text
        self.type_name = type_name
        if description is not None:
            self.description = description
        if nullable is not None:
            self.nullable = nullable
        if position is not None:
            self.position = position

    @property
    def name(self):
        """Gets the name of this ColumnInfo.  # noqa: E501

        Name of the column  # noqa: E501

        :return: The name of this ColumnInfo.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ColumnInfo.

        Name of the column  # noqa: E501

        :param name: The name of this ColumnInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def type_text(self):
        """Gets the type_text of this ColumnInfo.  # noqa: E501

        Data type in text format (e.g., 'string', 'int', 'array<string>')  # noqa: E501

        :return: The type_text of this ColumnInfo.  # noqa: E501
        :rtype: str
        """
        return self._type_text

    @type_text.setter
    def type_text(self, type_text):
        """Sets the type_text of this ColumnInfo.

        Data type in text format (e.g., 'string', 'int', 'array<string>')  # noqa: E501

        :param type_text: The type_text of this ColumnInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and type_text is None:  # noqa: E501
            raise ValueError("Invalid value for `type_text`, must not be `None`")  # noqa: E501

        self._type_text = type_text

    @property
    def type_name(self):
        """Gets the type_name of this ColumnInfo.  # noqa: E501

        Data type name in uppercase (e.g., 'STRING', 'INT')  # noqa: E501

        :return: The type_name of this ColumnInfo.  # noqa: E501
        :rtype: str
        """
        return self._type_name

    @type_name.setter
    def type_name(self, type_name):
        """Sets the type_name of this ColumnInfo.

        Data type name in uppercase (e.g., 'STRING', 'INT')  # noqa: E501

        :param type_name: The type_name of this ColumnInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and type_name is None:  # noqa: E501
            raise ValueError("Invalid value for `type_name`, must not be `None`")  # noqa: E501

        self._type_name = type_name

    @property
    def description(self):
        """Gets the description of this ColumnInfo.  # noqa: E501

        Description or comment for the column  # noqa: E501

        :return: The description of this ColumnInfo.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this ColumnInfo.

        Description or comment for the column  # noqa: E501

        :param description: The description of this ColumnInfo.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def nullable(self):
        """Gets the nullable of this ColumnInfo.  # noqa: E501

        Whether the column allows NULL values  # noqa: E501

        :return: The nullable of this ColumnInfo.  # noqa: E501
        :rtype: bool
        """
        return self._nullable

    @nullable.setter
    def nullable(self, nullable):
        """Sets the nullable of this ColumnInfo.

        Whether the column allows NULL values  # noqa: E501

        :param nullable: The nullable of this ColumnInfo.  # noqa: E501
        :type: bool
        """

        self._nullable = nullable

    @property
    def position(self):
        """Gets the position of this ColumnInfo.  # noqa: E501

        Position of the column in the table schema  # noqa: E501

        :return: The position of this ColumnInfo.  # noqa: E501
        :rtype: int
        """
        return self._position

    @position.setter
    def position(self, position):
        """Sets the position of this ColumnInfo.

        Position of the column in the table schema  # noqa: E501

        :param position: The position of this ColumnInfo.  # noqa: E501
        :type: int
        """

        self._position = position

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ColumnInfo):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ColumnInfo):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class WorkspaceTemplateVersion(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'template_id': 'str',
        'image_uri': 'str',
        'compute_configs': 'dict(str, str)',
        'artifacts': 'WorkspaceSystemArtifacts',
        'idle_termination_minutes': 'int',
        'id': 'str',
        'version': 'int',
        'creator_id': 'str',
        'creator_email': 'str',
        'created_at': 'datetime'
    }

    attribute_map = {
        'template_id': 'template_id',
        'image_uri': 'image_uri',
        'compute_configs': 'compute_configs',
        'artifacts': 'artifacts',
        'idle_termination_minutes': 'idle_termination_minutes',
        'id': 'id',
        'version': 'version',
        'creator_id': 'creator_id',
        'creator_email': 'creator_email',
        'created_at': 'created_at'
    }

    def __init__(self, template_id=None, image_uri=None, compute_configs=None, artifacts=None, idle_termination_minutes=None, id=None, version=None, creator_id=None, creator_email=None, created_at=None, local_vars_configuration=None):  # noqa: E501
        """WorkspaceTemplateVersion - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._template_id = None
        self._image_uri = None
        self._compute_configs = None
        self._artifacts = None
        self._idle_termination_minutes = None
        self._id = None
        self._version = None
        self._creator_id = None
        self._creator_email = None
        self._created_at = None
        self.discriminator = None

        self.template_id = template_id
        if image_uri is not None:
            self.image_uri = image_uri
        if compute_configs is not None:
            self.compute_configs = compute_configs
        if artifacts is not None:
            self.artifacts = artifacts
        if idle_termination_minutes is not None:
            self.idle_termination_minutes = idle_termination_minutes
        self.id = id
        self.version = version
        self.creator_id = creator_id
        self.creator_email = creator_email
        self.created_at = created_at

    @property
    def template_id(self):
        """Gets the template_id of this WorkspaceTemplateVersion.  # noqa: E501

        Parent workspace template ID  # noqa: E501

        :return: The template_id of this WorkspaceTemplateVersion.  # noqa: E501
        :rtype: str
        """
        return self._template_id

    @template_id.setter
    def template_id(self, template_id):
        """Sets the template_id of this WorkspaceTemplateVersion.

        Parent workspace template ID  # noqa: E501

        :param template_id: The template_id of this WorkspaceTemplateVersion.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and template_id is None:  # noqa: E501
            raise ValueError("Invalid value for `template_id`, must not be `None`")  # noqa: E501

        self._template_id = template_id

    @property
    def image_uri(self):
        """Gets the image_uri of this WorkspaceTemplateVersion.  # noqa: E501

        Docker image URI for this version  # noqa: E501

        :return: The image_uri of this WorkspaceTemplateVersion.  # noqa: E501
        :rtype: str
        """
        return self._image_uri

    @image_uri.setter
    def image_uri(self, image_uri):
        """Sets the image_uri of this WorkspaceTemplateVersion.

        Docker image URI for this version  # noqa: E501

        :param image_uri: The image_uri of this WorkspaceTemplateVersion.  # noqa: E501
        :type: str
        """

        self._image_uri = image_uri

    @property
    def compute_configs(self):
        """Gets the compute_configs of this WorkspaceTemplateVersion.  # noqa: E501

        Cloud-specific compute config mapping  # noqa: E501

        :return: The compute_configs of this WorkspaceTemplateVersion.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._compute_configs

    @compute_configs.setter
    def compute_configs(self, compute_configs):
        """Sets the compute_configs of this WorkspaceTemplateVersion.

        Cloud-specific compute config mapping  # noqa: E501

        :param compute_configs: The compute_configs of this WorkspaceTemplateVersion.  # noqa: E501
        :type: dict(str, str)
        """

        self._compute_configs = compute_configs

    @property
    def artifacts(self):
        """Gets the artifacts of this WorkspaceTemplateVersion.  # noqa: E501

        Artifacts configuration for this version  # noqa: E501

        :return: The artifacts of this WorkspaceTemplateVersion.  # noqa: E501
        :rtype: WorkspaceSystemArtifacts
        """
        return self._artifacts

    @artifacts.setter
    def artifacts(self, artifacts):
        """Sets the artifacts of this WorkspaceTemplateVersion.

        Artifacts configuration for this version  # noqa: E501

        :param artifacts: The artifacts of this WorkspaceTemplateVersion.  # noqa: E501
        :type: WorkspaceSystemArtifacts
        """

        self._artifacts = artifacts

    @property
    def idle_termination_minutes(self):
        """Gets the idle_termination_minutes of this WorkspaceTemplateVersion.  # noqa: E501

        Idle termination minutes for this version  # noqa: E501

        :return: The idle_termination_minutes of this WorkspaceTemplateVersion.  # noqa: E501
        :rtype: int
        """
        return self._idle_termination_minutes

    @idle_termination_minutes.setter
    def idle_termination_minutes(self, idle_termination_minutes):
        """Sets the idle_termination_minutes of this WorkspaceTemplateVersion.

        Idle termination minutes for this version  # noqa: E501

        :param idle_termination_minutes: The idle_termination_minutes of this WorkspaceTemplateVersion.  # noqa: E501
        :type: int
        """

        self._idle_termination_minutes = idle_termination_minutes

    @property
    def id(self):
        """Gets the id of this WorkspaceTemplateVersion.  # noqa: E501

        Unique version ID  # noqa: E501

        :return: The id of this WorkspaceTemplateVersion.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this WorkspaceTemplateVersion.

        Unique version ID  # noqa: E501

        :param id: The id of this WorkspaceTemplateVersion.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def version(self):
        """Gets the version of this WorkspaceTemplateVersion.  # noqa: E501

        Version number  # noqa: E501

        :return: The version of this WorkspaceTemplateVersion.  # noqa: E501
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this WorkspaceTemplateVersion.

        Version number  # noqa: E501

        :param version: The version of this WorkspaceTemplateVersion.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and version is None:  # noqa: E501
            raise ValueError("Invalid value for `version`, must not be `None`")  # noqa: E501

        self._version = version

    @property
    def creator_id(self):
        """Gets the creator_id of this WorkspaceTemplateVersion.  # noqa: E501

        User ID who created this version  # noqa: E501

        :return: The creator_id of this WorkspaceTemplateVersion.  # noqa: E501
        :rtype: str
        """
        return self._creator_id

    @creator_id.setter
    def creator_id(self, creator_id):
        """Sets the creator_id of this WorkspaceTemplateVersion.

        User ID who created this version  # noqa: E501

        :param creator_id: The creator_id of this WorkspaceTemplateVersion.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and creator_id is None:  # noqa: E501
            raise ValueError("Invalid value for `creator_id`, must not be `None`")  # noqa: E501

        self._creator_id = creator_id

    @property
    def creator_email(self):
        """Gets the creator_email of this WorkspaceTemplateVersion.  # noqa: E501

        Email of the creator  # noqa: E501

        :return: The creator_email of this WorkspaceTemplateVersion.  # noqa: E501
        :rtype: str
        """
        return self._creator_email

    @creator_email.setter
    def creator_email(self, creator_email):
        """Sets the creator_email of this WorkspaceTemplateVersion.

        Email of the creator  # noqa: E501

        :param creator_email: The creator_email of this WorkspaceTemplateVersion.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and creator_email is None:  # noqa: E501
            raise ValueError("Invalid value for `creator_email`, must not be `None`")  # noqa: E501

        self._creator_email = creator_email

    @property
    def created_at(self):
        """Gets the created_at of this WorkspaceTemplateVersion.  # noqa: E501

        When this version was created  # noqa: E501

        :return: The created_at of this WorkspaceTemplateVersion.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this WorkspaceTemplateVersion.

        When this version was created  # noqa: E501

        :param created_at: The created_at of this WorkspaceTemplateVersion.  # noqa: E501
        :type: datetime
        """
        if self.local_vars_configuration.client_side_validation and created_at is None:  # noqa: E501
            raise ValueError("Invalid value for `created_at`, must not be `None`")  # noqa: E501

        self._created_at = created_at

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, WorkspaceTemplateVersion):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, WorkspaceTemplateVersion):
            return True

        return self.to_dict() != other.to_dict()



from typing import Dict, List, Optional

from anyscale._private.models.model_base import ResultIterator
from anyscale._private.workload import WorkloadSDK
from anyscale.client.openapi_client.models.decorated_job_queue import DecoratedJobQueue
from anyscale.client.openapi_client.models.decoratedjobqueue_list_response import (
    DecoratedjobqueueListResponse,
)
from anyscale.client.openapi_client.models.job_queue_sort_directive import (
    JobQueueSortDirective,
)
from anyscale.client.openapi_client.models.list_response_metadata import (
    ListResponseMetadata,
)
from anyscale.client.openapi_client.models.resource_tag_resource_type import (
    ResourceTagResourceType,
)
from anyscale.client.openapi_client.models.session_state import SessionState
from anyscale.job_queue.models import JobQueueStatus


class PrivateJobQueueSDK(WorkloadSDK):
    """Internal SDK logic for Job Queue operations."""

    def list(
        self,
        *,
        job_queue_id: Optional[str] = None,
        name: Optional[str] = None,
        creator_id: Optional[str] = None,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
        cluster_status: Optional[SessionState] = None,
        tags_filter: Optional[Dict[str, List[str]]] = None,
        page_size: Optional[int] = None,
        max_items: Optional[int] = None,
        sorting_directives: Optional[List[JobQueueSortDirective]] = None,
    ) -> ResultIterator[JobQueueStatus]:
        """List job queues based on specified filters and pagination.

        If job_queue_id is provided, fetches only that specific job queue.
        """

        if job_queue_id is not None:
            raw = self._resolve_to_job_queue_model(job_queue_id=job_queue_id)

            def _fetch_single_page(
                _token: Optional[str],
            ) -> DecoratedjobqueueListResponse:
                # Only return data on the first call (token=None), simulate single-item page
                if _token is None and raw is not None:
                    results = [raw]
                    metadata = ListResponseMetadata(total=1, next_paging_token=None)
                else:
                    results = []
                    metadata = ListResponseMetadata(total=0, next_paging_token=None)

                return DecoratedjobqueueListResponse(
                    results=results, metadata=metadata,
                )

            return ResultIterator(
                page_token=None,
                max_items=1,  # Return the single fetched item
                fetch_page=_fetch_single_page,
                parse_fn=_parse_decorated_jq_to_status,
            )

        def _fetch_page(token: Optional[str]) -> DecoratedjobqueueListResponse:
            return self.client.list_job_queues(
                name=name,
                creator_id=creator_id,
                cloud=cloud,
                project=project,
                cluster_status=cluster_status,
                tags_filter=tags_filter,
                count=page_size,
                paging_token=token,
                sorting_directives=sorting_directives,
            )

        return ResultIterator(
            page_token=None,
            max_items=max_items,
            fetch_page=_fetch_page,
            parse_fn=_parse_decorated_jq_to_status,
        )

    def status(self, job_queue_id: str) -> JobQueueStatus:
        """Get the status and details for a specific job queue."""
        raw = self._resolve_to_job_queue_model(job_queue_id=job_queue_id)
        return _parse_decorated_jq_to_status(raw)

    def update(
        self,
        *,
        job_queue_id: Optional[str] = None,
        job_queue_name: Optional[str] = None,
        max_concurrency: Optional[int] = None,
        idle_timeout_s: Optional[int] = None,
    ) -> JobQueueStatus:
        """Update a job queue."""

        if max_concurrency is None and idle_timeout_s is None:
            raise ValueError("No fields to update")

        jq = self._resolve_to_job_queue_model(
            job_queue_id=job_queue_id, name=job_queue_name
        )

        assert jq.id is not None
        updated_jq = self.client.update_job_queue(
            job_queue_id=jq.id,
            max_concurrency=max_concurrency,
            idle_timeout_s=idle_timeout_s,
        )

        return _parse_decorated_jq_to_status(updated_jq)

    def add_tags(
        self,
        *,
        job_queue_id: Optional[str] = None,
        name: Optional[str] = None,
        tags: Dict[str, str],
    ) -> None:
        if not tags:
            raise ValueError("At least one tag must be provided.")

        if job_queue_id is not None:
            resource_id = job_queue_id
        else:
            if name is None:
                raise ValueError("Either 'job_queue_id' or 'name' must be provided.")
            jq = self._resolve_to_job_queue_model(job_queue_id=None, name=name)
            if jq.id is None:
                raise RuntimeError(f"Job queue with name '{name}' has no ID.")
            resource_id = jq.id

        self.client.upsert_resource_tags(
            ResourceTagResourceType.JOB_QUEUE, resource_id, tags
        )

    def remove_tags(
        self,
        *,
        job_queue_id: Optional[str] = None,
        name: Optional[str] = None,
        keys: List[str],
    ) -> None:
        if not keys:
            raise ValueError("At least one tag key must be provided.")

        if job_queue_id is not None:
            resource_id = job_queue_id
        else:
            if name is None:
                raise ValueError("Either 'job_queue_id' or 'name' must be provided.")
            jq = self._resolve_to_job_queue_model(job_queue_id=None, name=name)
            if jq.id is None:
                raise RuntimeError(f"Job queue with name '{name}' has no ID.")
            resource_id = jq.id

        self.client.delete_resource_tags(
            ResourceTagResourceType.JOB_QUEUE, resource_id, keys
        )

    def _resolve_to_job_queue_model(
        self, *, job_queue_id: Optional[str] = None, name: Optional[str] = None,
    ) -> DecoratedJobQueue:
        """Finds the specific Job Queue API model by ID or name.
        """
        if job_queue_id is None and name is None:
            raise ValueError("Either 'job_queue_id' or 'name' must be provided.")

        if job_queue_id:
            job_queue = self.client.get_job_queue(job_queue_id)
            if job_queue is None:
                raise ValueError(f"Job Queue with ID '{job_queue_id}' not found.")
            return job_queue
        else:
            job_queues_response = self.client.list_job_queues(name=name, count=1)
            if len(job_queues_response.results) == 0:
                raise ValueError(f"Job Queue with name '{name}' not found.")
            return job_queues_response.results[0]

    def list_tags(
        self, *, job_queue_id: Optional[str] = None, name: Optional[str] = None,
    ) -> Dict[str, str]:
        """List tags for a job queue as a key/value mapping."""
        if job_queue_id is not None:
            resource_id = job_queue_id
        else:
            jq = self._resolve_to_job_queue_model(job_queue_id=None, name=name)
            if jq.id is None:
                raise RuntimeError(f"Job queue with name '{name}' has no ID.")
            resource_id = jq.id
        records = self.client.list_resource_tags(
            ResourceTagResourceType.JOB_QUEUE, resource_id
        )
        return {r.key: r.value for r in records if r and r.key is not None}


def _parse_decorated_jq_to_status(decorated_jq: DecoratedJobQueue) -> JobQueueStatus:
    """Helper to convert API model to SDK model."""

    if decorated_jq.id is None or decorated_jq.current_job_queue_state is None:
        raise ValueError("Job Queue ID or state is missing.")

    return JobQueueStatus(
        id=decorated_jq.id,
        name=decorated_jq.name,
        state=decorated_jq.current_job_queue_state,
        creator_email=decorated_jq.creator_email,
        project_id=decorated_jq.project_id,
        created_at=decorated_jq.created_at,
        max_concurrency=decorated_jq.max_concurrency,
        idle_timeout_s=decorated_jq.idle_timeout_sec,
        creator_id=decorated_jq.creator_id,
        cloud_id=decorated_jq.cloud_id,
        user_provided_id=decorated_jq.user_provided_id,
        execution_mode=decorated_jq.execution_mode,
        total_jobs=decorated_jq.total_jobs,
        active_jobs=decorated_jq.active_jobs,
        successful_jobs=decorated_jq.successful_jobs,
        failed_jobs=decorated_jq.failed_jobs,
    )



# pylint:disable=private-import
import builtins
from contextlib import contextmanager
import copy
import hashlib
import logging
import os
from pathlib import Path
import tempfile
from typing import (
    Any,
    Callable,
    Dict,
    Generator,
    List,
    Optional,
    Tuple,
    TYPE_CHECKING,
    Union,
)
from urllib.parse import urlparse

import click

from anyscale.cli_logger import BlockLogger
from anyscale.client.openapi_client import ComputeTemplate, DecoratedCloudResource
from anyscale.client.openapi_client.api.default_api import DefaultApi
from anyscale.client.openapi_client.models.cloud_providers import CloudProviders
from anyscale.client.openapi_client.models.user_info import UserInfo
from anyscale.sdk.anyscale_client.api.default_api import DefaultApi as SDKDefaultApi
from anyscale.util import is_anyscale_workspace
from anyscale.utils.ray_utils import zip_directory  # type: ignore
from anyscale.utils.workload_types import Workload


if TYPE_CHECKING:
    pass

logger = logging.getLogger(__name__)

# TODO(austin): refactor to read requirements.txt and .skip_packages_tracking from s3 or gcs directly.
# Default cluster storage directory.
CLUSTER_STORAGE_DIR = os.environ.get(
    "ANYSCALE_CLUSTER_STORAGE_DIR", "/mnt/cluster_storage"
)

# Location of the directory containing workspace configurations.
WORKSPACE_CONF_DIR = os.environ.get(
    "ANYSCALE_WORKSPACE_CONF_DIR", os.path.join(CLUSTER_STORAGE_DIR, ".anyscale")
)

# Location of the workspace-managed requirements.txt file that is automatically populated.
# This is not always guaranteed to exist within a workspace.
WORKSPACE_REQUIREMENTS_FILE_PATH = os.path.join(WORKSPACE_CONF_DIR, "requirements.txt")

# Feature flags for pip dependency tracking.
SKIP_PACKAGES_TRACKING_PATH = os.path.join(
    WORKSPACE_CONF_DIR, ".skip_packages_tracking"
)


def is_workspace_dependency_tracking_disabled() -> bool:
    """Returns True if the workspace dependency tracking is disabled.

    Enabled iff:
        - ANYSCALE_WORKSPACE_DYNAMIC_DEPENDENCY_TRACKING env var is set to 1.
        - ANYSCALE_SKIP_PYTHON_DEPENDENCY_TRACKING env var is not set to 1.
        - No file exists at SKIP_PACKAGES_TRACKING_PATH.
    """
    # NOTE(edoakes): The environment variable is evaluated here instead of in the global scope
    # so it can easily be overwritten for testing.
    enabled = (
        os.environ.get("ANYSCALE_WORKSPACE_DYNAMIC_DEPENDENCY_TRACKING", "0") == "1"
    )
    return (
        not enabled
        or os.environ.get("ANYSCALE_SKIP_PYTHON_DEPENDENCY_TRACKING", "0") == "1"
        or os.path.exists(SKIP_PACKAGES_TRACKING_PATH)
    )


def _upload_file_to_google_cloud_storage(file: str, bucket: str, object_name: str):
    try:
        from google.cloud import (  # noqa: PLC0415 - codex_reason("gpt5.2", "optional GCP storage dependency")
            storage,
        )

    except Exception:  # noqa: BLE001
        raise click.ClickException(
            "Could not upload file to Google Storage. Could not import the Google Storage Python API via `from google.cloud import storage`.  Please check your installation or try running `pip install --upgrade google-cloud-storage`."
        )
    try:
        storage_client = storage.Client()
        bucket_obj = storage_client.bucket(bucket)
        blob = bucket_obj.blob(object_name)
        blob.upload_from_filename(file)
    except Exception as e:  # noqa: BLE001
        raise click.ClickException(
            f"Failed to upload the working directory to Google Cloud Storage. Error {e!r}"
            "Please validate you have exported cloud credentials with the correct write permissions and the intended bucket exists in your Cloud Storage account. "
            "If you do not desire to upload your working directory, please set your working directory to a public remote URI or remove the runtime_environment from your job or service yaml."
        ) from e


def _upload_file_to_s3(file: str, bucket: str, object_key: str):
    try:
        import boto3  # noqa: PLC0415 - codex_reason("gpt5.2", "optional AWS SDK dependency")
        import botocore.config  # noqa: PLC0415 - codex_reason("gpt5.2", "optional AWS SDK dependency")
    except Exception:  # noqa: BLE001
        raise click.ClickException(
            "Could not upload file to S3: Could not import the Amazon S3 Python API via `import boto3`.  Please check your installation or try running `pip install boto3`."
        )
    try:
        s3_client = boto3.client(
            "s3", config=botocore.config.Config(signature_version="s3v4")
        )
        s3_client.upload_file(file, bucket, object_key)
    except Exception as e:  # noqa: BLE001
        raise click.ClickException(
            f"Failed to upload the working directory to S3. Error {e!r}"
            "Please validate you have exported cloud credentials with the correct write permissions and the intended bucket exists in your S3 account. "
            "If you do not desire to upload your working directory, please set your working directory to a public remote URI or remove the runtime_environment from your job or service yaml."
        ) from e


def _upload_file_to_azure_storage(file: str, upload_path: str, object_name: str) -> str:
    """Upload a file to Azure Blob Storage using smart_open (same library Ray uses for downloads).

    Args:
        file: Local file path to upload
        upload_path: Azure storage path (abfss://container@account.dfs.core.windows.net/path)
        object_name: Name of the object to upload

    Returns:
        str: The final uploaded filepath (azure://container/path/filename)

    Raises:
        click.ClickException: If upload fails or required packages are missing
    """
    try:
        from azure.identity import (  # noqa: PLC0415 - codex_reason("gpt5.2", "optional Azure identity dependency")
            DefaultAzureCredential,
        )
        from azure.storage.blob import (  # noqa: PLC0415 - codex_reason("gpt5.2", "optional Azure storage dependency")
            BlobServiceClient,
        )
        import smart_open  # noqa: PLC0415 - codex_reason("gpt5.2", "optional smart_open dependency for Azure")
    except ImportError as e:  # noqa: BLE001
        raise click.ClickException(
            f"Could not upload file to Azure Storage. Missing required packages: {e}. "
            "Please install: pip install smart-open[azure] azure-storage-blob azure-identity"
        )

    parsed = urlparse(upload_path)
    # smart_open uses 'azure://' scheme, not 'abfss://'
    # Typical forms we want to support:
    #   - abfss://container@account.dfs.core.windows.net/path
    #   - azure://container/path
    #
    # For abfss, we can derive both the container and storage account from the URL.
    # For azure://, we only know the container from the URL, so we require the
    # storage account name to be provided via environment variable.
    if parsed.scheme == "abfss":
        container_and_account = parsed.netloc
        # Example: "container@account.dfs.core.windows.net"
        container_name = container_and_account.split("@")[0]
        storage_account_name: str = container_and_account.split("@")[1].split(".")[0]
    elif parsed.scheme == "azure":
        # Example: azure://container/path
        container_name = parsed.netloc

        # We need the storage account name to construct the BlobServiceClient.
        # Require it to be passed via environment variable so users can control
        # which account is used when only given an azure:// URI.
        # Use the same standard name used elsewhere in the codebase.
        storage_account_name = os.environ.get("AZURE_STORAGE_ACCOUNT", "")
        if not storage_account_name:
            raise click.ClickException(
                "When using an 'azure://' upload_path, you must set either "
                "AZURE_STORAGE_ACCOUNT to the Azure Storage account name."
            )
    else:
        raise click.ClickException(
            f"Unsupported Azure URI scheme for upload_path: {upload_path!r}"
        )

    # Construct azure:// URL for smart_open using the same logic as S3/GCS
    final_uploaded_filepath = f"azure://{container_name}/{object_name}"

    try:
        credential = DefaultAzureCredential()
        blob_service_client = BlobServiceClient(
            account_url=f"https://{storage_account_name}.blob.core.windows.net",
            credential=credential,
        )
        transport_params = {"client": blob_service_client}
    except Exception as e:  # noqa: BLE001
        raise click.ClickException(
            f"Failed to create Azure BlobServiceClient: {e!r}. "
            "Please ensure Azure credentials are available (managed identity or environment variables)."
        ) from e

    try:
        with smart_open.open(
            final_uploaded_filepath, "wb", transport_params=transport_params
        ) as fout, builtins.open(file, "rb") as fin:
            fout.write(fin.read())
    except Exception as e:  # noqa: BLE001
        raise click.ClickException(
            f"Failed to upload the working directory to Azure Storage. Error {e!r} "
            "Please validate you have exported Azure credentials with the correct write permissions. "
            "If you do not desire to upload your working directory, please set your working directory to a public remote URI or remove the runtime_environment from your job or service yaml."
        ) from e
    return final_uploaded_filepath


def _get_remote_storage_object_name(upload_path, upload_filename):
    # Strip leading slash, otherwise bucket will create a new directory called "/".
    object_name = os.path.join(urlparse(upload_path).path, upload_filename).lstrip("/")
    return object_name


def _upload_file_to_remote_storage(
    source_file: str, upload_path: str, upload_filename: str
):
    parsed_upload_path = urlparse(upload_path)
    service = parsed_upload_path.scheme
    bucket = parsed_upload_path.netloc
    object_name = _get_remote_storage_object_name(upload_path, upload_filename)
    if service in ("abfss", "azure"):
        return _upload_file_to_azure_storage(source_file, upload_path, object_name)
    if service == "s3":
        _upload_file_to_s3(source_file, bucket, object_key=object_name)
    if service == "gs":
        _upload_file_to_google_cloud_storage(
            source_file, bucket, object_name=object_name
        )
    final_uploaded_filepath = os.path.join(upload_path, upload_filename)
    try:
        from smart_open import (  # noqa: PLC0415 - codex_reason("gpt5.2", "optional smart_open dependency for remote storage")
            open,  # noqa: A004 - claude_comment("claude-opus-4-5", "smart_open.open intentionally shadows builtin for file-like API")
        )

        open(final_uploaded_filepath)
    except Exception as e:  # noqa: BLE001
        raise click.ClickException(
            f"Could not open uploaded file, maybe something went wrong while uploading: {e}."
        )

    return final_uploaded_filepath


def is_dir_remote_uri(target_dir: str) -> bool:
    parsed = urlparse(target_dir)
    return bool(parsed.scheme)


@contextmanager
def zip_local_dir(
    path: str, *, excludes: Optional[List[str]] = None
) -> Generator[Tuple[str, bytes, str], None, None]:
    """Packs the local directory into a temporary zip file.

    After the context manager exits, the file is deleted.

    Yields: (temp_path, file_contents, content_hash).
    """
    with tempfile.TemporaryDirectory() as temp_dir:
        zip_file_path = os.path.join(temp_dir, "anyscale_generated_working_dir.zip")
        zip_directory(
            path,
            excludes=excludes if excludes is not None else [],
            output_path=zip_file_path,
            # Ray requires remote Zip URIs to consist of a single top-level directory when unzipped.
            include_parent_dir=True,
        )
        zip_file_bytes = Path(zip_file_path).read_bytes()
        yield zip_file_path, zip_file_bytes, hashlib.md5(zip_file_bytes).hexdigest()


def upload_and_rewrite_working_dir(
    runtime_env_json: Dict[str, Any],
    upload_file_to_remote_storage_fn: Callable[
        [str, str, str], str
    ] = _upload_file_to_remote_storage,
) -> Dict[str, Any]:
    """Upload a local working_dir and rewrite the working_dir field with the destination remote URI.

    After uploading, deletes the "upload_path" field because it is no longer used and is not a valid
    OSS runtime env field.
    """
    if runtime_env_json.get("working_dir") is None:
        return runtime_env_json

    working_dir = runtime_env_json["working_dir"]
    if is_dir_remote_uri(working_dir):
        # The working dir is a remote URI already
        return runtime_env_json

    upload_path = runtime_env_json["upload_path"]
    excludes = runtime_env_json.get("excludes")
    with zip_local_dir(working_dir, excludes=excludes) as (zip_file_path, _, hash_val):
        uploaded_zip_file_name = f"_anyscale_pkg_{hash_val}.zip"
        final_uploaded_filepath = upload_file_to_remote_storage_fn(
            zip_file_path, upload_path, uploaded_zip_file_name,
        )

    final_runtime_env = runtime_env_json.copy()
    final_runtime_env["working_dir"] = final_uploaded_filepath
    del final_runtime_env["upload_path"]
    return final_runtime_env


def override_runtime_env_config(
    runtime_env: Optional[Dict[str, Any]],
    anyscale_api_client: Union[DefaultApi, SDKDefaultApi],
    api_client: DefaultApi,
    workload_type: Workload,
    compute_config_id: Optional[str],
    log: BlockLogger,
) -> Optional[Dict[str, Any]]:
    """Override the working_dir, upload_path, and pip fields for a runtime_env.

    When running inside a workspace:
        1. Autopopulates the `working_dir` with the current directory.
        2. Autopopulates the `pip` field with the contents of the workspace-managed
           `requirements.txt` file.

    If the working_dir is a local path, will upload the contents to cloud storage and
    replace the field with the resulting remote URI. The upload_path can be specified
    in the runtime_env, else it will be auto-populated for the cloud.
    """

    existing_runtime_env = autopopulate_runtime_env_for_workspace(
        runtime_env=runtime_env, log=log
    )

    if not existing_runtime_env:
        return {}
    elif not existing_runtime_env.get("working_dir"):
        return existing_runtime_env

    working_dir = existing_runtime_env.get("working_dir", "")
    upload_path = existing_runtime_env.get("upload_path")

    if not is_dir_remote_uri(working_dir):
        if upload_path is not None:
            # If upload_path is specified
            # we back up the current working dir to the specified path
            new_runtime_env = upload_and_rewrite_working_dir(existing_runtime_env)
        elif is_anyscale_workspace() and "ANYSCALE_SESSION_ID" in os.environ:
            # If submitting job v2 from workspaces and no upload_path is specified,
            # we back up the current workspace content into S3
            cluster_id = os.environ["ANYSCALE_SESSION_ID"]

            decorated_cluster = api_client.get_decorated_cluster_api_v2_decorated_sessions_cluster_id_get(
                cluster_id
            ).result
            cloud_id = decorated_cluster.cloud.id

            workspace_id = os.environ["ANYSCALE_EXPERIMENTAL_WORKSPACE_ID"]

            new_runtime_env = infer_upload_path_and_rewrite_working_dir(
                api_client=api_client,
                existing_runtime_env=existing_runtime_env,
                workload_type=workload_type,
                cloud_id=cloud_id,
                log=log,
                workspace_id=workspace_id,
            )
        else:
            compute_template: ComputeTemplate = anyscale_api_client.get_compute_template(
                compute_config_id
            ).result
            cloud_id = compute_template.config.cloud_id
            new_runtime_env = infer_upload_path_and_rewrite_working_dir(
                api_client=api_client,
                existing_runtime_env=existing_runtime_env,
                workload_type=workload_type,
                cloud_id=cloud_id,
                log=log,
            )

        return new_runtime_env
    else:
        return existing_runtime_env


def parse_dot_env_file(dot_env_bytes: bytes) -> Dict[str, str]:
    """Parse a .env file and return a dictionary of key-value pairs."""
    dot_env = dot_env_bytes.split(b"\x00")
    ret = {}
    for kv in dot_env:
        if len(kv) == 0:
            # skip empty lines
            continue
        segs = kv.split(b"=", 1)
        if len(segs) == 2:
            key, value = segs
            try:
                ret[key.decode().strip()] = value.decode().strip()
            except UnicodeDecodeError:
                logger.error(f"Failed to decode env var entry: {kv!r}")
        else:
            logger.error(f"Invalid env var entry: {kv!r}")
    return ret


def parse_requirements_file(path: str) -> Optional[List[str]]:
    """TODO: add comment."""
    requirements_file = Path(path)
    if requirements_file.is_file():
        lines = requirements_file.read_text().strip().split("\n")
        parsed_requirements = []
        for line in lines:
            # Strip comments and whitespace
            hash_idx = line.find("#")
            content = (line[:hash_idx] if hash_idx != -1 else line).strip()
            if content:
                parsed_requirements.append(content)
    else:
        parsed_requirements = None

    return parsed_requirements


def autopopulate_runtime_env_for_workspace(
    runtime_env: Optional[Dict[str, Any]],
    log: BlockLogger,
    *,
    requirements_file_path: str = WORKSPACE_REQUIREMENTS_FILE_PATH,
) -> Optional[Dict[str, Any]]:
    """Autopopulates fields of the runtime_env for commands run in a workspace.

    Fields populated (if not specified by the user):
        - working_dir: set to ".".
        - pip: set to the contents of the workspace-managed requirements.txt file.
          If the file does not exist, this field will not be set.

    No-op if called outside of a workspace.
    """
    if not is_anyscale_workspace():
        return runtime_env

    if not runtime_env:
        runtime_env = {}

    if not runtime_env.get("working_dir"):
        runtime_env["working_dir"] = "."
        log.info("working_dir is not specified, using the current local directory.")

    # Workspaces maintains a `requirements.txt` file in a well-known location.
    # By default, populate the runtime environment with the contents of this file.
    # If the user passes any "pip" dependencies, do not overwrite them.
    if not is_workspace_dependency_tracking_disabled():
        parsed_requirements = parse_requirements_file(requirements_file_path)
        if parsed_requirements:
            if runtime_env.get("pip"):
                log.info(
                    "Not including workspace-tracked dependencies because "
                    "'pip' field is specified in the runtime_env."
                )
            elif runtime_env.get("conda"):
                log.info(
                    "Not including workspace-tracked dependencies because "
                    "'conda' field is specified in the runtime_env."
                )
            else:
                runtime_env["pip"] = parsed_requirements
                log.info("Including workspace pip dependencies.")

    return runtime_env


def infer_upload_path_and_rewrite_working_dir(
    *,
    api_client: DefaultApi,
    existing_runtime_env: Dict[str, Any],
    cloud_id: str,
    workload_type: Workload,
    log: BlockLogger,
    workspace_id: Optional[str] = None,
) -> Dict[str, Any]:
    """
    Push working_dir to remote bucket and rewrite the working_dir field with the destination uri

    If the upload_path is not specified by the user, we will get the bucket name based on the cloud.
    We then rewrite the working_dir to the remote uri path
    so that the launched service will read from remote bucket directly.

    For Workspaces:
        The remote path: [s3, gs, abfss]://{bucket_name}/{org_id}/{cloud_id}/workspace_snapshots/{workspace_id}/{workload_type}/{backup_zip}
    Otherwise:
        The remote path: [s3, gs, abfss]://{bucket_name}/{org_id}/{cloud_id}/{workload_type}/{backup_zip}
        workload_type=[jobs, scheduled_jobs, services]
    """

    all_cloud_resources = api_client.get_cloud_resources_api_v2_clouds_cloud_id_resources_get(
        cloud_id=cloud_id,
    ).results

    if len(all_cloud_resources) == 0:
        raise click.ClickException(f"No cloud resources found for cloud {cloud_id}.")

    primary_cloud_resource: DecoratedCloudResource = all_cloud_resources[0]

    bucket_name = _get_cloud_storage_bucket_name_from_cloud_resource(
        primary_cloud_resource
    )

    if primary_cloud_resource.provider == CloudProviders.AWS:
        protocol = "s3"
    elif primary_cloud_resource.provider == CloudProviders.GCP:
        protocol = "gs"
    elif primary_cloud_resource.provider == CloudProviders.AZURE:
        if not bucket_name:
            raise ValueError(
                f"Cloud {cloud_id} does not have an Azure storage bucket configured. "
                "Please configure a storage bucket for this cloud."
            )

        # Parse the scheme from the bucket name (e.g., abfss://container@account.dfs.core.windows.net)
        parsed_bucket = urlparse(bucket_name)
        if not parsed_bucket.scheme:
            raise click.ClickException(
                f"Invalid Azure storage bucket name format: {bucket_name}. "
                "Expected format: abfss://container-name@storage-account.dfs.core.windows.net"
            )
        protocol = str(parsed_bucket.scheme)
        # Extract storage account name for Ray runtime environment
        # Format: abfss://container@storage-account.dfs.core.windows.net
        netloc_str = str(parsed_bucket.netloc) if parsed_bucket.netloc else ""
        if not netloc_str:
            raise click.ClickException(
                f"Invalid Azure storage bucket name format: {bucket_name}. "
                "Expected format: abfss://container-name@storage-account.dfs.core.windows.net"
            )
        storage_account_name = netloc_str.split("@")[1].split(".")[0]
        # For Azure, use netloc (container@account.dfs.core.windows.net) for path construction
        bucket_name = netloc_str
    else:
        raise click.ClickException(
            f"Currently launching jobs or services from workspaces in a {primary_cloud_resource.provider} cloud is not supported. "
            "Please contact Anyscale support for more info."
        )

    new_runtime_env = copy.deepcopy(existing_runtime_env)
    working_dir_path = Path(new_runtime_env["working_dir"]).absolute()
    log.info(f"Uploading local working_dir from '{working_dir_path}'.")

    org_id = _get_organization_id(api_client)
    if workspace_id:
        new_runtime_env[
            "upload_path"
        ] = f"{protocol}://{bucket_name}/{org_id}/{cloud_id}/workspace_snapshots/{workspace_id}/{workload_type}"
    else:
        new_runtime_env[
            "upload_path"
        ] = f"{protocol}://{bucket_name}/{org_id}/{cloud_id}/{workload_type}"

    # For Azure, set AZURE_STORAGE_ACCOUNT environment variable for Ray to download working_dir
    if primary_cloud_resource.provider == CloudProviders.AZURE and storage_account_name:
        if "env_vars" not in new_runtime_env:
            new_runtime_env["env_vars"] = {}
        new_runtime_env["env_vars"]["AZURE_STORAGE_ACCOUNT"] = storage_account_name
        log.info(
            f"Setting AZURE_STORAGE_ACCOUNT={storage_account_name} for Ray runtime environment."
        )

    new_runtime_env = upload_and_rewrite_working_dir(new_runtime_env)
    return new_runtime_env


def _get_organization_id(api_client: DefaultApi):
    user_info: UserInfo = (api_client.get_user_info_api_v2_userinfo_get().result)
    orgs = user_info.organizations
    return orgs[0].id


def _get_cloud_storage_bucket_name_from_cloud_resource(
    cloud_resource: DecoratedCloudResource,
) -> Optional[str]:
    """
    If the cloud resource has an associated AWS S3, Google Storage or Azure Storage bucket, we return its name.

    Please note that this is only for v2 clouds where customers have their own object storage.
    """
    if cloud_resource.object_storage and cloud_resource.object_storage.bucket_name:
        return cloud_resource.object_storage.bucket_name
    else:
        return None



from dataclasses import dataclass, field
from datetime import datetime
from typing import ClassVar, Dict, List, Optional, Union

from anyscale._private.models import ImageURI, ModelBase
from anyscale._private.models.model_base import ModelEnum
from anyscale.compute_config.models import (
    compute_config_type_from_dict,
    ComputeConfig,
    ComputeConfigType,
    MultiResourceComputeConfig,
)


class WorkspaceState(ModelEnum):
    """Possible states for a workspace."""

    STARTING = "STARTING"
    UPDATING = "UPDATING"
    RUNNING = "RUNNING"
    TERMINATING = "TERMINATING"
    TERMINATED = "TERMINATED"
    ERRORED = "ERRORED"
    UNKNOWN = "UNKNOWN"

    __docstrings__: ClassVar[Dict[str, str]] = {
        "STARTING": "The workspace is starting up.",
        "UPDATING": "The workspace is updating.",
        "RUNNING": "The workspace is running.",
        "TERMINATING": "The workspace is terminating.",
        "TERMINATED": "The workspace is terminated.",
        "ERRORED": "The workspace is in an error state.",
        "UNKNOWN": "The workspace state",
    }


class WorkspaceSortField(ModelEnum):
    """Fields available for sorting workspaces."""

    STATUS = "STATUS"
    CREATED_AT = "CREATED_AT"
    LATEST_STARTED_AT = "LATEST_STARTED_AT"

    __docstrings__: ClassVar[Dict[str, str]] = {
        "STATUS": "Sort by workspace status (active first by default).",
        "CREATED_AT": "Sort by creation timestamp.",
        "LATEST_STARTED_AT": "Sort by most recent start time.",
    }


class WorkspaceSortOrder(ModelEnum):
    """Enum for sort order directions."""

    ASC = "ASC"
    DESC = "DESC"

    __docstrings__: ClassVar[Dict[str, str]] = {
        "ASC": "Sort in ascending order.",
        "DESC": "Sort in descending order.",
    }


@dataclass(frozen=True)
class UpdateWorkspaceConfig(ModelBase):
    """Configuration options for updating a workspace."""

    __doc_py_example__ = """\
from anyscale.workspace.models import UpdateWorkspaceConfig

config = UpdateWorkspaceConfig(
    name="new-workpsace-name",
    idle_termination_minutes=220,
    env_vars={"key": "value"},
    requirements="/tmp/requirements.txt",
)
"""

    __doc_yaml_example__ = """\
name: new-workspace-name
idle_termination_minutes: 220
env_vars:
    key: value
requirements: /tmp/requirements.txt
tags:
    team: mlops
    env: prod
"""

    name: Optional[str] = field(
        default=None, metadata={"docstring": "The name of the workspace"}
    )

    def _validate_name(self, name: Optional[str]):
        if name is not None and not isinstance(name, str):
            raise TypeError("'name' must be a string.")

    image_uri: Optional[str] = field(
        default=None,
        metadata={
            "docstring": "URI of an existing image. Exclusive with `containerfile`."
        },
    )

    def _validate_image_uri(self, image_uri: Optional[str]):
        if image_uri is not None and not isinstance(image_uri, str):
            raise TypeError(f"'image_uri' must be an str but it is {type(image_uri)}.")
        if image_uri is not None:
            ImageURI.from_str(image_uri)

    containerfile: Optional[str] = field(
        default=None,
        repr=False,
        metadata={
            "docstring": "The file path to a containerfile that will be built into an image before running the workload. Exclusive with `image_uri`."
        },
    )

    def _validate_containerfile(self, containerfile: Optional[str]):
        if containerfile is not None and not isinstance(containerfile, str):
            raise TypeError("'containerfile' must be a string.")

    tags: Optional[Dict[str, str]] = field(
        default=None, metadata={"docstring": "Tags to associate with the workspace."},
    )

    def _validate_tags(self, tags: Optional[Dict[str, str]]):
        if tags is None:
            return
        if not isinstance(tags, dict):
            raise TypeError("'tags' must be a Dict[str, str].")
        for k, v in tags.items():
            if not isinstance(k, str) or not isinstance(v, str):
                raise TypeError("'tags' must be a Dict[str, str].")

    compute_config: Union[ComputeConfigType, Dict, str, None] = field(
        default=None,
        metadata={
            "docstring": "The name of an existing registered compute config (including version) or an inlined ComputeConfig object."
        },
    )

    def _validate_compute_config(
        self, compute_config: Union[ComputeConfigType, Dict, str, None]
    ) -> Union[None, str, ComputeConfigType]:
        if compute_config is None or isinstance(compute_config, str):
            return compute_config

        if isinstance(compute_config, dict):
            compute_config = compute_config_type_from_dict(compute_config)
        if not isinstance(compute_config, (ComputeConfig, MultiResourceComputeConfig)):
            raise TypeError(
                "'compute_config' must be a string, ComputeConfig, MultiResourceComputeConfig, or corresponding dict"
            )

        return compute_config

    idle_termination_minutes: Optional[int] = field(
        default=None,
        metadata={
            "docstring": "the time in minutes after which the workspace is terminated when idle."
        },
    )

    def _validate_idle_termination_minutes(
        self, idle_termination_minutes: Optional[int]
    ):
        if idle_termination_minutes is not None and not isinstance(
            idle_termination_minutes, int
        ):
            raise ValueError("'idle_termination_minutes' must be an int")

    requirements: Optional[Union[str, List[str]]] = field(
        default=None,
        repr=False,
        metadata={
            "docstring": "A list of pip requirements or a path to a `requirements.txt` file for the workload."
        },
    )

    def _validate_requirements(self, requirements: Optional[Union[str, List[str]]]):
        if requirements is None or isinstance(requirements, str):
            return

        if not isinstance(requirements, list) or not all(
            isinstance(r, str) for r in requirements
        ):
            raise TypeError(
                "'requirements' must be a string (file path) or list of strings."
            )

    env_vars: Optional[Dict[str, str]] = field(
        default=None,
        repr=True,
        metadata={
            "docstring": "A dictionary of environment variables that will be set for the workload."
        },
    )

    def _validate_env_vars(self, env_vars: Optional[Dict[str, str]]):
        if env_vars is not None and (
            not isinstance(env_vars, dict)
            or not all(
                isinstance(k, str) and isinstance(v, str) for k, v in env_vars.items()
            )
        ):
            raise TypeError("'env_vars' must be a Dict[str, str].")

    registry_login_secret: Optional[str] = field(
        default=None,
        repr=False,
        metadata={
            "docstring": "A name or identifier of the secret containing credentials to authenticate to the docker registry hosting the image. "
            "This can only be used when 'image_uri' is specified and the image is not hosted on Anyscale."
        },
    )

    def _validate_registry_login_secret(self, registry_login_secret: Optional[str]):
        if registry_login_secret is not None and not isinstance(
            registry_login_secret, str
        ):
            raise TypeError("'registry_login_secret' must be a string.")

    ray_version: Optional[str] = field(
        default=None,
        metadata={
            "docstring": "The Ray version (X.Y.Z) specified for this image specified by either an image URI or a containerfile. If you don't specify a Ray version, Anyscale defaults to the latest Ray version available at the time of the Anyscale CLI/SDK release."
        },
    )

    def _validate_ray_version(self, ray_version: Optional[str]):
        if ray_version is not None and not isinstance(ray_version, str):
            raise TypeError("'ray_version' must be a string.")
        if ray_version:
            vs = ray_version.split(".")
            if len(vs) != 3:
                raise ValueError(
                    f"Invalid Ray version format: {ray_version}. Must be in the format 'X.Y.Z'."
                )


@dataclass(frozen=True)
class WorkspaceConfig(UpdateWorkspaceConfig):
    """Configuration options for a workspace."""

    __doc_py_example__ = """\
from anyscale.workspace.models import WorkspaceConfig

config = WorkspaceConfig(
    name="my-workpsace",
    idle_termination_minutes=220,
    env_vars={"key": "value"},
    requirements="/tmp/requirements.txt",
)
"""

    __doc_yaml_example__ = """\
name: my-workspace
idle_termination_minutes: 220
env_vars:
    key: value
requirements: /tmp/requirements.txt
tags:
  team: mlops
  env: prod
"""

    project: Optional[str] = field(
        default=None,
        metadata={
            "docstring": "The project for the workload. If not provided, the default project for the cloud will be used."
        },
    )

    def _validate_project(self, project: Optional[str]):
        if project is not None and not isinstance(project, str):
            raise TypeError("'project' must be a string.")

    cloud: Optional[str] = field(
        default=None,
        metadata={
            "docstring": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used."
        },
    )

    def _validate_cloud(self, cloud: Optional[str]):
        if cloud is not None and not isinstance(cloud, str):
            raise TypeError("'cloud' must be a string.")


@dataclass(frozen=True)
class Workspace(ModelBase):
    """Workspace information including metadata and state."""

    __doc_py_example__ = """\
import anyscale
from anyscale.workspace.models import Workspace

workspace: Workspace = anyscale.workspace.get(name="workspace-name")
first_workspace: Workspace = next(anyscale.workspace.list(max_items=1), None)
"""

    id: str = field(metadata={"docstring": "Unique identifier of the workspace."})

    def _validate_id(self, id: str):  # noqa: A002
        if not isinstance(id, str):
            raise ValueError("The workspace id must be a string.")

    name: str = field(metadata={"docstring": "The name of the workspace."})

    def _validate_name(self, name: str):
        if not isinstance(name, str):
            raise ValueError("The workspace name must be a string.")

    state: WorkspaceState = field(
        metadata={"docstring": "The current state of the workspace."}
    )

    def _validate_state(self, state: WorkspaceState):
        WorkspaceState.validate(state)

    project_id: Optional[str] = field(
        default=None,
        metadata={"docstring": "Project identifier that owns this workspace."},
    )

    def _validate_project_id(self, project_id: Optional[str]):
        if project_id is not None and not isinstance(project_id, str):
            raise ValueError("The project id must be a string if provided.")

    cloud_id: Optional[str] = field(
        default=None,
        metadata={"docstring": "Cloud identifier where the workspace is running."},
    )

    def _validate_cloud_id(self, cloud_id: Optional[str]):
        if cloud_id is not None and not isinstance(cloud_id, str):
            raise ValueError("The cloud id must be a string if provided.")

    creator_id: Optional[str] = field(
        default=None,
        metadata={"docstring": "Identifier of the user who created the workspace."},
    )

    def _validate_creator_id(self, creator_id: Optional[str]):
        if creator_id is not None and not isinstance(creator_id, str):
            raise ValueError("The creator id must be a string if provided.")

    creator_email: Optional[str] = field(
        default=None,
        metadata={"docstring": "Email address of the user who created the workspace."},
    )

    def _validate_creator_email(self, creator_email: Optional[str]):
        if creator_email is not None and not isinstance(creator_email, str):
            raise ValueError("The creator email must be a string if provided.")

    created_at: Optional[datetime] = field(
        default=None,
        metadata={"docstring": "Timestamp when the workspace was created."},
    )

    def _validate_created_at(self, created_at: Optional[datetime]):
        if created_at is not None and not isinstance(created_at, datetime):
            raise ValueError("The created_at field must be a datetime if provided.")

    last_started_at: Optional[datetime] = field(
        default=None,
        metadata={"docstring": "Timestamp when the workspace was last started."},
    )

    def _validate_last_started_at(self, last_started_at: Optional[datetime]):
        if last_started_at is not None and not isinstance(last_started_at, datetime):
            raise ValueError(
                "The last_started_at field must be a datetime if provided."
            )

    cluster_id: Optional[str] = field(
        default=None, metadata={"docstring": "Cluster identifier for the workspace."},
    )

    def _validate_cluster_id(self, cluster_id: Optional[str]):
        if cluster_id is not None and not isinstance(cluster_id, str):
            raise ValueError("The cluster id must be a string if provided.")

    config: Optional[Union[WorkspaceConfig, Dict]] = field(
        default=None,
        repr=False,
        metadata={
            "docstring": "Full workspace configuration. Only populated for get() operations, not list()."
        },
    )

    def _validate_config(
        self, config: Optional[Union[WorkspaceConfig, Dict]]
    ) -> Optional[WorkspaceConfig]:
        if config is None:
            return None
        if isinstance(config, dict):
            config = WorkspaceConfig.from_dict(config)
        if not isinstance(config, WorkspaceConfig):
            raise TypeError("'config' must be a WorkspaceConfig or corresponding dict.")
        return config



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class OrganizationUserGroupsCollaborator(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'name': 'str',
        'org_id': 'str',
        'created_at': 'datetime',
        'updated_at': 'datetime',
        'permission_level': 'OrganizationPermissionLevel',
        'members': 'list[MiniUser]',
        'member_count': 'int'
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'org_id': 'org_id',
        'created_at': 'created_at',
        'updated_at': 'updated_at',
        'permission_level': 'permission_level',
        'members': 'members',
        'member_count': 'member_count'
    }

    def __init__(self, id=None, name=None, org_id=None, created_at=None, updated_at=None, permission_level=None, members=None, member_count=None, local_vars_configuration=None):  # noqa: E501
        """OrganizationUserGroupsCollaborator - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._name = None
        self._org_id = None
        self._created_at = None
        self._updated_at = None
        self._permission_level = None
        self._members = None
        self._member_count = None
        self.discriminator = None

        self.id = id
        self.name = name
        self.org_id = org_id
        self.created_at = created_at
        self.updated_at = updated_at
        if permission_level is not None:
            self.permission_level = permission_level
        if members is not None:
            self.members = members
        self.member_count = member_count

    @property
    def id(self):
        """Gets the id of this OrganizationUserGroupsCollaborator.  # noqa: E501

        The user group ID  # noqa: E501

        :return: The id of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this OrganizationUserGroupsCollaborator.

        The user group ID  # noqa: E501

        :param id: The id of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def name(self):
        """Gets the name of this OrganizationUserGroupsCollaborator.  # noqa: E501

        The user group name  # noqa: E501

        :return: The name of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this OrganizationUserGroupsCollaborator.

        The user group name  # noqa: E501

        :param name: The name of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def org_id(self):
        """Gets the org_id of this OrganizationUserGroupsCollaborator.  # noqa: E501


        :return: The org_id of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :rtype: str
        """
        return self._org_id

    @org_id.setter
    def org_id(self, org_id):
        """Sets the org_id of this OrganizationUserGroupsCollaborator.


        :param org_id: The org_id of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and org_id is None:  # noqa: E501
            raise ValueError("Invalid value for `org_id`, must not be `None`")  # noqa: E501

        self._org_id = org_id

    @property
    def created_at(self):
        """Gets the created_at of this OrganizationUserGroupsCollaborator.  # noqa: E501


        :return: The created_at of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this OrganizationUserGroupsCollaborator.


        :param created_at: The created_at of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :type: datetime
        """
        if self.local_vars_configuration.client_side_validation and created_at is None:  # noqa: E501
            raise ValueError("Invalid value for `created_at`, must not be `None`")  # noqa: E501

        self._created_at = created_at

    @property
    def updated_at(self):
        """Gets the updated_at of this OrganizationUserGroupsCollaborator.  # noqa: E501


        :return: The updated_at of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this OrganizationUserGroupsCollaborator.


        :param updated_at: The updated_at of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :type: datetime
        """
        if self.local_vars_configuration.client_side_validation and updated_at is None:  # noqa: E501
            raise ValueError("Invalid value for `updated_at`, must not be `None`")  # noqa: E501

        self._updated_at = updated_at

    @property
    def permission_level(self):
        """Gets the permission_level of this OrganizationUserGroupsCollaborator.  # noqa: E501

        The organization-level role for this group  # noqa: E501

        :return: The permission_level of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :rtype: OrganizationPermissionLevel
        """
        return self._permission_level

    @permission_level.setter
    def permission_level(self, permission_level):
        """Sets the permission_level of this OrganizationUserGroupsCollaborator.

        The organization-level role for this group  # noqa: E501

        :param permission_level: The permission_level of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :type: OrganizationPermissionLevel
        """

        self._permission_level = permission_level

    @property
    def members(self):
        """Gets the members of this OrganizationUserGroupsCollaborator.  # noqa: E501

        List of group members  # noqa: E501

        :return: The members of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :rtype: list[MiniUser]
        """
        return self._members

    @members.setter
    def members(self, members):
        """Sets the members of this OrganizationUserGroupsCollaborator.

        List of group members  # noqa: E501

        :param members: The members of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :type: list[MiniUser]
        """

        self._members = members

    @property
    def member_count(self):
        """Gets the member_count of this OrganizationUserGroupsCollaborator.  # noqa: E501

        Total number of members  # noqa: E501

        :return: The member_count of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :rtype: int
        """
        return self._member_count

    @member_count.setter
    def member_count(self, member_count):
        """Sets the member_count of this OrganizationUserGroupsCollaborator.

        Total number of members  # noqa: E501

        :param member_count: The member_count of this OrganizationUserGroupsCollaborator.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and member_count is None:  # noqa: E501
            raise ValueError("Invalid value for `member_count`, must not be `None`")  # noqa: E501

        self._member_count = member_count

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrganizationUserGroupsCollaborator):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrganizationUserGroupsCollaborator):
            return True

        return self.to_dict() != other.to_dict()
